////////////////////////////////////////////////////////////////////////////////
//
//  Copyright 2013 by Michael A. Morris, dba M. A. Morris & Associates
//
//  All rights reserved. The source code contained herein is publicly released
//  under the terms and conditions of the GNU Lesser Public License. No part of
//  this source code may be reproduced or transmitted in any form or by any
//  means, electronic or mechanical, including photocopying, recording, or any
//  information storage and retrieval system in violation of the license under
//  which the source code is released.
//
//  The source code contained herein is free; it may be redistributed and/or
//  modified in accordance with the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either version 2.1 of
//  the GNU Lesser General Public License, or any later version.
//
//  The source code contained herein is freely released WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
//  PARTICULAR PURPOSE. (Refer to the GNU Lesser General Public License for
//  more details.)
//
//  A copy of the GNU Lesser General Public License should have been received
//  along with the source code contained herein; if not, a copy can be obtained
//  by writing to:
//
//  Free Software Foundation, Inc.
//  51 Franklin Street, Fifth Floor
//  Boston, MA  02110-1301 USA
//
//  Further, no use of this source code is permitted in any form or means
//  without inclusion of this banner prominently in any derived works.
//
//  Michael A. Morris
//  Huntsville, AL
//
////////////////////////////////////////////////////////////////////////////////

`timescale 1ns / 1ps

//////////////////////////////////////////////////////////////////////////////////
// Company:         Michael A. Morris 
// Engineer:        M. A. Morris & Associates
// 
// Create Date:     15:57:32 09/28/2013 
// Design Name:     ChoChi FPGA-based 6502 Development Board
// Module Name:     ChoChi_MemoryIF. 
// Project Name:    C:\XProjects\ISE10.1i\M65C02A 
// Target Devices:  SRAM-based FPGAs: XC3S50-xTQ144; XC3S50A-4VQ100 
// Tool versions:   Xilinx ISE 10.1i
//    
// Description: 
//
// Dependencies: 
//
// Revision: 
//
//  0.01    13I28   MAM     File Created 
//
// Additional Comments: 
//
////////////////////////////////////////////////////////////////////////////////

module SRAM_StbGen (
    input   Rst,
    
    input   Clk0,
    input   Clk1,
    
    input   Stb,
    output  nOut
);

////////////////////////////////////////////////////////////////////////////////
//
//  Implementation
//

// ODDR2: Output Double Data Rate Output Register with Set, Reset
// and Clock Enable.
// Spartan-3E/3A
// Xilinx HDL Libraries Guide, version 10.1.2

ODDR2   #(
            .DDR_ALIGNMENT("NONE"), // Sets output alignment: "NONE", "C0", "C1"
            .INIT(1'b1),            // Sets initial state of the Q output: 1, 0
            .SRTYPE("ASYNC")        // Specifies "SYNC" or "ASYNC" set/reset
        ) FF (
            .Q(nOut),               // 1-bit DDR output data
            .C0(Clk0),              // 1-bit clock input
            .C1(Clk1),              // 1-bit clock input
            .CE(Stb),               // 1-bit clock enable input
            .D0(1'b0),              // 1-bit data input (associated with C0)
            .D1(1'b1),              // 1-bit data input (associated with C1)
            .R(1'b0),               // 1-bit reset input
            .S(Rst)                 // 1-bit set input
        );

// End of ODDR2_inst instantiation

endmodule
