`timescale 1ns / 1ps
////////////////////////////////////////////////////////////////////////////////
// Company:         Michael A. Morris 
// Engineer:        M. A. Morris & Associates
//
// Create Date:     16:48:03 09/28/2013
// Design Name:     ChoChi_SRAM_IF
// Module Name:     C:/XProjects/ISE10.1i/M65C02A/Src/tb_ChoChi_SRAM_IF.v
// Project Name:    M65C02A
// Target Device:   XC3S50E-TQ100  
// Tool versions:   ISE10.1i SP3
//  
// Description: 
//
// Verilog Test Fixture created by ISE for module: ChoChi_SRAM_IF
//
// Dependencies:
// 
// Revision:
//
//  0.01    13I28   MAM     File Created
//
// Additional Comments:
// 
////////////////////////////////////////////////////////////////////////////////

module tb_ChoChi_SRAM_IF;

parameter pMemAddr = 11;

// Inputs
reg Rst;
reg Clk, Clk2x;

//  SRAM Interface - FPGA Internal

reg     Sel;
reg     WE;
reg     RE;
reg     [(pMemAddr - 1):0] A;
reg     [ 7:0] DO;
wire    [ 7:0] DI;

//  SRAM Interface - External Memory

wire    nRAM_CE;
wire    nRAM_OE;
wire    nRAM_WE;
wire    [(pMemAddr - 1):0] RAM_A;
tri     [7:0] RAM_DQ;

//  Simulation Variables

integer i = 0;
integer j = 0;

wire    dClk;

wire    [7:0] RAM_DO;
reg     [7:0] RAM_DI, Pattern;

// Instantiate the Unit Under Test (UUT)

ChoChi_SRAM_IF  #(
                    .pMemAddr(pMemAddr)
                ) uut (
                    .Rst(Rst), 
                    .Clk(dClk),
                    .Clk2x(Clk2x),
                    
                    .Sel(Sel), 
                    .WE(WE), 
                    .RE(RE), 
                    .A(A), 
                    .DO(DO), 
                    .DI(DI), 

                    .nRAM_CE(nRAM_CE), 
                    .nRAM_OE(nRAM_OE), 
                    .nRAM_WE(nRAM_WE), 
                    .RAM_A(RAM_A), 
                    .RAM_DQ(RAM_DQ)
                );
                
//  Instantiate RAM Module

M65C02_RAM  #(
                .pAddrSize(pMemAddr),
                .pDataSize(8),
                .pFileName("Src/M65C02_RAM.txt")
            ) RAM (
                .Clk(Clk2x),
                .Ext(1'b0),     // 1 cycle memory; asynchronous SRAM
                .ZP(1'b1),
                .WE(~nRAM_WE),
                .AI(RAM_A[(pMemAddr - 1):0]),
                .DI(RAM_DQ),
                .DO(RAM_DO)
            );

initial begin
    // Initialize Inputs
    Rst     = 1;
    Clk     = 1;
    Clk2x   = 1;
    
    Sel     = 0;
    WE      = 0;
    RE      = 0;
    A       = 0;
    DO      = 0;
    
    i       = 0;
    j       = 0;
    RAM_DI  = 0;
    Pattern = 0;

    // Wait 100 ns for global reset to finish
    #101 Rst = 0;

    // Add stimulus here
    
    @(posedge Clk) #1;
    
    for(i = 0; i < (1 << pMemAddr); i = i + 1) begin
        Wr_SRAM(1'b1, i[(pMemAddr - 1):0], ~i[7:0]);
    end
    
    Sel = 0;
    
    @(posedge Clk) #1;
    @(posedge Clk) #1;
    @(posedge Clk) #1;
    @(posedge Clk) #1;

    for(i = 0; i < (1 << pMemAddr); i = i + 1) begin
        Rd_SRAM(1'b1, i[(pMemAddr - 1):0], RAM_DI);
    end
    
    Sel = 0;
    
    @(posedge Clk) #1;
    @(posedge Clk) #1;
    @(posedge Clk) #1;
    @(posedge Clk) #1;

    //  Perform Walking 1s test on Data Bus
    
    for(i = 0; i < (1 << pMemAddr); i = i + 1) begin
        for(j = 0; j < 8; j = j + 1) begin
            Pattern = (1 << j);
            Wr_SRAM(1'b1, i[(pMemAddr - 1):0], Pattern);
            Rd_SRAM(1'b1, i[(pMemAddr - 1):0], RAM_DI);
            
            if(RAM_DI != Pattern) begin
                $display("  Error(%h): Expected %h, Detected %h",
                         i, Pattern, RAM_DI                      );

                Sel = 0;
                
                @(posedge Clk) #1;
                @(posedge Clk) #1;
                @(posedge Clk) #1;
                @(posedge Clk) #1;

                $stop;
            end
        end
    end
    
    //  End of Simulation
    
    Sel = 0;
    
    @(posedge Clk) #1;
    @(posedge Clk) #1;
    @(posedge Clk) #1;
    @(posedge Clk) #1;

    $stop;
end

////////////////////////////////////////////////////////////////////////////////
//
//  Clocks
//

always #10 Clk   = ~Clk;
always #5  Clk2x = ~Clk2x;

assign #0.01 dClk = Clk;

////////////////////////////////////////////////////////////////////////////////
//
//  Simulation structures
//

assign RAM_DQ = ((~nRAM_OE) ? RAM_DO : 8'bZ);

////////////////////////////////////////////////////////////////////////////////
//
//  Simulation Tasks and Functions
//

task Wr_SRAM;
    input   SRAM_Sel;
    input   [(pMemAddr - 1):0] Addr;
    input   [7:0] Wr_Data;

begin
    if(SRAM_Sel) begin
        Sel = SRAM_Sel;
        WE  = 1;
        A   = Addr;
        DO  = Wr_Data;
        @(posedge Clk) #1 WE = 0;
    end else begin
        Sel = 0;
        A   = Addr;
        DO  = Wr_Data;
        @(posedge Clk) #1;
    end
end

endtask

task Rd_SRAM;
    input   SRAM_Sel;
    input   [(pMemAddr - 1):0] Addr;
    output  [7:0] Rd_Data;

begin
    if(SRAM_Sel) begin
        Sel = SRAM_Sel;
        RE  = 1;
        A   = Addr;
        @(posedge Clk) #1.1;
            RE = 0;
            Rd_Data = DI;
    end else begin
        Sel = 0;
        A   = Addr;
        @(posedge Clk) #1.1;
            Rd_Data = DI;
    end
end

endtask

endmodule

